#region Using directives

using System;
using System.Collections.Generic;
using System.Text;

#endregion

namespace LspEvents
{
    public class Lsp
    {
        public delegate void LspEventHandler(string msg);

        // The Lsp can send the following events:
        public event LspEventHandler FullyCongested;
        public event LspEventHandler GettingCongested;
        public event LspEventHandler CannotForwardData;

        // Internal state data.
        private string LspName;
        private int SourceNode;
        private int DestinationNode;
        private int MaximumBandwidth; // units: megabits per second
        private string PathElements;
        bool LspCanForwardData;
        
        private int _requiredBandwidth;
        public int RequiredBandwidth
        {
        	get { return _requiredBandwidth; }
        	set { _requiredBandwidth = value; }
        }        

        public Lsp()
        {
        }

        public Lsp(string lspName, int sourceNode, int destinationNode, 
        string path, int requiredBandwidth, int maximumBandwidth)
        {
        	#region Constructor
        	LspName = lspName;
        	SourceNode = sourceNode;
        	DestinationNode = destinationNode;
        	PathElements = path;
        	RequiredBandwidth = requiredBandwidth;
        	MaximumBandwidth = maximumBandwidth;
        	LspCanForwardData = true;
        	#endregion
        }

        public void ForwardSomeData(int requiredBandwidth)
        {
        	#region Use the LSP
        	RequiredBandwidth += requiredBandwidth;

            // If the Lsp is congested, fire the FullyCongested event.
            if (LspCanForwardData)
            {
            	// Check for the worst case to begin with
                if (MaximumBandwidth - RequiredBandwidth <= 0 &&
            	    FullyCongested != null)
            	{
            		LspCanForwardData = false;
            		FullyCongested(LspName + " is congested, can't forward data.");
            	}
            	else
            		if (MaximumBandwidth - RequiredBandwidth <= 50
            		    && GettingCongested != null)
            		{
	                   GettingCongested("Congestion is looming. Max. bandwidth on " + LspName + ": " + 
            		                 MaximumBandwidth + ". Required bandwidth: " +  
            		                 RequiredBandwidth);
    	            }
            		else
            			Console.WriteLine("Currently forwarding data at bandwidth" +
            			                  " {0} Mbps", RequiredBandwidth);
            }
            else
            {
                // If the Lsp is congested, fire the Congested event.
                if (CannotForwardData != null)
                	CannotForwardData(LspName + " is disabled, can't forward data.");
            }
            #endregion
        }
    }
}
